#pragma once
#include "unassignable.h"
#include <string>
#include <list>
#include "wimp.h"
#include "window.h"

namespace rov2
{

class CMenu;

class CTask : public CUnassignable
{
public:
  CTask(const std::string& rksTaskName);
  virtual ~CTask(void);

  /*! Enters the main poll loop */
  void Poll();

  /*! Stops the current poll loop */
  inline void Stop() { m_bExecuting = false; }

  /*! Called whenever a Wimp_Poll event has occurred
  \param weEventCode The event code
  \param rwbWimpBlock The WIMP poll result block
  \return True if the event was handled
  */
  virtual bool PollEvent(eWimpEvent weEventCode, tWimpBlock& rwbWimpBlock);

  /*! Called whenever the desktop is going to shut down. Return false to abort the shutdown */
  virtual bool PreQuit() { return true; }

  /*! Gets the current task handle
  \return The current task handle
  */
  inline const tTaskHandle TaskHandle() { return m_thMyTask; }

  /*! Register a window
  \param kpwWindow Pointer to the window
  */
  void RegisterWindow(const CWindow* kpwWindow);

  /*! Deregister a window
  \param kpwWindow Pointer to the window
  */
  void DeregisterWindow(const CWindow* kpwWindow);

  /*! Finds a window based on the window handle
  \param kwhWindow The window handle to look for
  \return The window that the handle refers to, or NULL if it cannot be found
  */
  const CWindow* FindWindow(const tWindowHandle kwhWindow);

  /*! Acknowledges a message
  \param rumbMessageBlock The message block to acknowledge. Note that this block is corrupted.
  */
  void AcknowledgeMessage(tUserMessageBlock& rumbMessageBlock);

  /*! Acknowledges a message
  \param kwmaUserAction The user action to reply with
  \param rumbMessageBlock The message block to acknowledge. Note that this block is corrupted.
  */
  void AcknowledgeMessage(const eWimpMessageAction kwmaUserAction, tUserMessageBlock& rumbMessageBlock);

  /*! Gets the current task
  \return The current task
  */
  static inline CTask* CurrentTask() { return sm_pctCurrent; }

  /*! Set the current menu
  \param kpmMenu The menu which needs to be opened
  \param rlLocation The location at which it needs to be opened
  */
  void OpenMenu(CMenu* kpmMenu, tLocation& rlLocation);
private:
  /*! The current task handle*/
  tTaskHandle m_thMyTask;

  /*! True if the poll loop should be executing */
  bool m_bExecuting;

  /*! The list of windows known by the task */
  std::list<const CWindow*> m_lkpwWindows;

  /*! The current task */
  static CTask* sm_pctCurrent;

  /*! The current menu */
  CMenu* m_kpmCurrentMenu;

  /*! The current menu location */
  tLocation m_lCurrentMenuLocation;
};

}

